@@ -12,7 +12,7 @@ import pango
 from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib, i18n
 
-from tortoisehg.hgtk import csinfo, gdialog
+from tortoisehg.hgtk import csinfo, gdialog, textview, gtklib
 
 keep = i18n.keepgettext()
 
@@ -58,8 +58,9 @@ class BackoutDialog(gdialog.GDialog):
         self.cmsgframe = frame
 
         ## message text area
-        self.logview = gtk.TextView(buffer=None)
-        self.logview.set_editable(True)
+        accelgroup = gtk.AccelGroup()
+        self.add_accel_group(accelgroup)
+        self.logview = textview.UndoableTextView(accelgroup=accelgroup)
         fontcomment = hglib.getfontconfig()['fontcomment']
         self.logview.modify_font(pango.FontDescription(fontcomment))
         self.buf = self.logview.get_buffer()
@@ -71,7 +72,7 @@ class BackoutDialog(gdialog.GDialog):
         msgvbox.pack_start(scrolledwindow)
 
         ## tooltips
-        self.tips = gtk.Tooltips()
+        self.tips = gtklib.Tooltips()
         self.tips.set_tip(frame,
                 _('Commit message text for new changeset that reverses the'
                 '  effect of the change being backed out.'))
@@ -81,6 +82,8 @@ class BackoutDialog(gdialog.GDialog):
         ## use English backout message option
         self.eng_msg = gtk.CheckButton(_('Use English backout message'))
         self.eng_msg.connect('toggled', self.eng_msg_toggled)
+        engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
+        self.eng_msg.set_active(engmsg)
         hbox.pack_start(self.eng_msg, False, False)
 
         ## merge after backout
@@ -116,14 +119,10 @@ class BackoutDialog(gdialog.GDialog):
             self.cmd.set_result(_('Failed to backout'), style='error')
 
     def load_settings(self):
-        checked = self.settings.get_value('english', False, True)
-        self.eng_msg.set_active(checked)
         checked = self.settings.get_value('merge', True, True)
         self.merge_button.set_active(checked)
 
     def store_settings(self):
-        checked = self.eng_msg.get_active()
-        self.settings.set_value('english', checked)
         checked = self.merge_button.get_active()
         self.settings.set_value('merge', checked)
         self.settings.write()
