@@ -19,6 +19,8 @@ from tortoisehg.util import shlib, hglib, paths
 
 from tortoisehg.hgtk import csinfo, gdialog, gtklib, hgcmd, statusbar
 
+MODE_FILEPATS = 1 # mirrored from history.py
+
 class ChangeSet(gdialog.GWindow):
     'GTK+ based dialog for displaying repository logs'
     def __init__(self, ui, repo, cwd, pats, opts, stbar=None):
@@ -145,10 +147,10 @@ class ChangeSet(gdialog.GWindow):
             gview = self.graphview
             if oldother:
                 gview.model.clear_parents()
+                gview.queue_draw()
             elif self.otherparent:
                 gview.model.set_parent(ctx.rev(), parent)
-            gview.hide()
-            gview.show()
+                gview.queue_draw()
 
         # update dialog title
         self.set_title(title)
@@ -166,7 +168,10 @@ class ChangeSet(gdialog.GWindow):
 
         self._filelist.clear()
         self._filelist.append(('*', _('[All Files]'), ''))
-        modified, added, removed = self.repo.status(parent, ctx.node())[:3]
+        try:
+            modified, added, removed = self.repo.status(parent, ctx.node())[:3]
+        except error.LookupError:
+            modified, added, removed = [], [], []
         selrow = None
         for f in modified:
             if f in pats:
@@ -194,6 +199,16 @@ class ChangeSet(gdialog.GWindow):
 
     def load_patch_details(self, patchfile):
         'Load specified patch details into buffer and file list'
+        pf = open(patchfile)
+        self.load_patch_details_from_file_object(pf, patchfile)
+        
+    def load_patch_details_from_file_object(self, pf, patchfile, isTemp=False):
+        """ Load patch details into buffer and file list
+        :param pf: open file object
+        :param patchfile: path and name of patch file
+        :param isTemp: if True, then pf is a temporary file 
+            and patchfile does not exist
+        """
         self._filelist.clear()
         self._filelist.append(('*', _('[All Files]'), ''))
 
@@ -201,7 +216,11 @@ class ChangeSet(gdialog.GWindow):
         self.currev = -1
         self.curphunks = {}
         self.curpatch = patchfile
-        pf = open(self.curpatch)
+        if isTemp:
+            # pf is a temporary, so update panel cache while we can
+            patch_ctx = csinfo.patchctx(patchfile, self.repo, patchHandle=pf)
+            self.summarypanel.update(patch_ctx, self.patchstyle)
+            pf.seek(0)
         def get_path(a, b):
             type = (a == '/dev/null') and 'A' or 'M'
             type = (b == '/dev/null') and 'R' or type
@@ -269,6 +288,11 @@ class ChangeSet(gdialog.GWindow):
         adj.set_value(0)
         adj = self.diffscroller.get_hadjustment()
         adj.set_value(0)
+        adj = self._filelist_tree.get_vadjustment()
+        adj.set_value(0)
+        adj = self._filelist_tree.get_hadjustment()
+        adj.set_value(0)
+        self._filelist_tree.columns_autosize()
 
     def generate_change_header(self):
         self.summarypanel.update(self.currev, self.csetstyle)
@@ -295,7 +319,7 @@ class ChangeSet(gdialog.GWindow):
         self.issuedict.clear()
         for m in self.bodyre.finditer(desc):
             a, b = m.span()
-            if a > pos:
+            if a >= pos:
                 buf.insert(eob, desc[pos:a])
                 pos = b
             groups = m.groups()
@@ -343,7 +367,6 @@ class ChangeSet(gdialog.GWindow):
         tags, lines = self.prepare_diff(lines, offset, wfile)
         for l in lines:
             buf.insert(eob, l)
-
         # inserts the tags
         for name, p0, p1 in tags:
             i0 = buf.get_iter_at_offset(p0)
@@ -422,7 +445,7 @@ class ChangeSet(gdialog.GWindow):
                 tag = 'red'
                 l = hglib.diffexpand(l)
             else:
-                tag = 'black'
+                tag = 'normal'
                 l = hglib.diffexpand(l)
             l = l+"\n"
             length = len(l.decode('utf-8'))
@@ -578,7 +601,7 @@ class ChangeSet(gdialog.GWindow):
         def widget_func(widget, item, markups):
             def linkwidget(revnum, revid, summary, highlight=None, branch=None):
                 # revision label
-                opts = dict(underline='single', color='blue')
+                opts = dict(underline='single', color=gtklib.BLUE)
                 if highlight:
                     opts['weight'] = 'bold'
                 rev = '%s (%s)' % (gtklib.markup(revnum, **opts),
@@ -590,7 +613,7 @@ class ChangeSet(gdialog.GWindow):
                 # summary & branch label
                 sum = gtklib.markup(summary)
                 if branch:
-                    sum = gtklib.markup(branch, color='black',
+                    sum = gtklib.markup(branch, color=gtklib.NORMAL,
                         background=gtklib.PGREEN) + ' ' + sum
                 sumlabel = gtk.Label()
                 sumlabel.set_markup(sum)
@@ -793,7 +816,7 @@ class ChangeSet(gdialog.GWindow):
         tag_table = self._buffer.get_tag_table()
 
         tag_table.add(make_texttag('diff', font=self.rawfonts['fontdiff']))
-        tag_table.add(make_texttag('blue', foreground='blue'))
+        tag_table.add(make_texttag('blue', foreground=gtklib.BLUE))
         if self.colorstyle == 'background':
             tag_table.add(make_texttag('red',
                                        paragraph_background=gtklib.PRED))
@@ -805,21 +828,21 @@ class ChangeSet(gdialog.GWindow):
         else:
             tag_table.add(make_texttag('red', foreground=gtklib.DRED))
             tag_table.add(make_texttag('green', foreground=gtklib.DGREEN))
-        tag_table.add(make_texttag('black', foreground='black'))
+        tag_table.add(make_texttag('normal', foreground=gtklib.NORMAL))
         tag_table.add(make_texttag('greybg',
-                                   paragraph_background='grey',
+                                   paragraph_background=gtklib.CHANGE_HEADER,
                                    weight=pango.WEIGHT_BOLD))
-        tag_table.add(make_texttag('yellowbg', background='yellow'))
+        tag_table.add(make_texttag('yellowbg', background=gtklib.YELLOW))
 
-        issuelink_tag = make_texttag('issuelink', foreground='blue',
+        issuelink_tag = make_texttag('issuelink', foreground=gtklib.BLUE,
                                      underline=pango.UNDERLINE_SINGLE)
         issuelink_tag.connect('event', self.issuelink_event)
         tag_table.add(issuelink_tag)
-        urllink_tag = make_texttag('urllink', foreground='blue',
+        urllink_tag = make_texttag('urllink', foreground=gtklib.BLUE,
                                      underline=pango.UNDERLINE_SINGLE)
         urllink_tag.connect('event', self.urllink_event)
         tag_table.add(urllink_tag)
-        csetlink_tag = make_texttag('csetlink', foreground='blue',
+        csetlink_tag = make_texttag('csetlink', foreground=gtklib.BLUE,
                                     underline=pango.UNDERLINE_SINGLE)
         csetlink_tag.connect('event', self.csetlink_event)
         tag_table.add(csetlink_tag)
@@ -927,8 +950,9 @@ class ChangeSet(gdialog.GWindow):
             return
         try:
             q = Queue.Queue()
-            hglib.hgcmd_toq(q, 'cat', '--rev',
-                str(self.currev), '--output', hglib.fromutf(result), self.curfile)
+            hglib.hgcmd_toq(q, False, ('cat', '--rev',
+                            str(self.currev), '--output', hglib.fromutf(result),
+                            self.curfile))
         except (util.Abort, IOError), e:
             gdialog.Prompt(_('Unable to save file'), str(e), self).run()
 
@@ -991,8 +1015,7 @@ class ChangeSet(gdialog.GWindow):
             explorer.filtercombo.set_active(1)
             explorer.filterentry.set_text(fname)
             explorer.filterbar.get_button('custom').set_active(True)
-            explorer.filter_entry_activated(explorer.filterentry,
-                                            explorer.filtercombo)
+            explorer.activate_filter(path, MODE_FILEPATS)
         else:
             # Else launch our own glog instance
             from tortoisehg.hgtk import history
@@ -1007,7 +1030,7 @@ class ChangeSet(gdialog.GWindow):
                  (self.curfile, rev))
         if dialog.run() == gtk.RESPONSE_NO:
             return
-        cmdline = ['hg', 'revert', '--verbose', '--rev', str(rev), self.curfile]
+        cmdline = ['hg', 'revert', '--verbose', '--rev', str(rev), '--', self.curfile]
         dlg = hgcmd.CmdDialog(cmdline)
         dlg.run()
         dlg.hide()
