@@ -83,7 +83,7 @@ class CloneDialog(gdialog.GDialog):
 
         # layout table for fixed options
         self.table = table = gtklib.LayoutTable()
-        vbox.pack_start(table, True, True, 2)
+        vbox.pack_start(table, False, False, 2)
 
         ## comboentry for source paths
         self.srclist, srccombo = createcombo(self.srcpath,
@@ -115,7 +115,7 @@ class CloneDialog(gdialog.GDialog):
 
         # expander for advanced options
         self.expander = expander = gtk.Expander(_('Advanced options'))
-        vbox.pack_start(expander, True, True, 2)
+        vbox.pack_start(expander, False, False, 2)
 
         # layout table for advanced options
         table = gtklib.LayoutTable()
@@ -171,19 +171,23 @@ class CloneDialog(gdialog.GDialog):
     def get_action_map(self):
         return {gtk.RESPONSE_OK: self.clone}
 
-    def switch_to(self, normal, *args):
+    def switch_to(self, normal, working, *args):
         self.table.set_sensitive(normal)
         self.expander.set_sensitive(normal)
         self.buttons['clone'].set_property('visible', normal)
         self.buttons['cancel'].set_property('visible', normal)
         if normal:
             self.buttons['cancel'].grab_focus()
+        if working:
+            self.set_resizable(True)
+            self.vbox.set_child_packing(self.cmd, True, True, 6, gtk.PACK_START)
 
     def command_done(self, returncode, useraborted, src, dest):
         self.add_src_to_recent(src)
         self.add_dest_to_recent(dest)
         if returncode == 0:
-            shlib.shell_notify([dest])
+            if dest:
+                shlib.shell_notify([dest])
             self.cmd.set_result(_('Cloned successfully'), style='ok')
         elif useraborted:
             self.cmd.set_result(_('Canceled updating'), style='error')
@@ -206,6 +210,8 @@ class CloneDialog(gdialog.GDialog):
                      initial=entry.get_text().strip(), title=title).run()
         if res:
             entry.set_text(res)
+            entry.grab_focus()
+            entry.set_position(-1)
 
     def bundle_clicked(self, button, title, entry):
         path = entry.get_text().strip()
@@ -327,6 +333,7 @@ class CloneDialog(gdialog.GDialog):
         cmdline.append('--verbose')
         cmdline.append(hglib.fromutf(src))
         if dest:
+            cmdline.append('--')
             cmdline.append(hglib.fromutf(dest))
 
         # start cloning
