@@ -19,11 +19,14 @@ import time
 from mercurial import hg, util, patch, cmdutil, extensions
 
 from tortoisehg.util.i18n import _
-from tortoisehg.util import shlib, hglib
+from tortoisehg.util import shlib, hglib, i18n
+
+keep = i18n.keepgettext()
 
 from tortoisehg.hgtk.status import GStatus, FM_STATUS, FM_CHECKED
 from tortoisehg.hgtk.status import FM_PATH, FM_PATH_UTF8
-from tortoisehg.hgtk import csinfo, gtklib, thgconfig, gdialog, hgcmd, thgmq
+from tortoisehg.hgtk import csinfo, gtklib, thgconfig, gdialog, hgcmd
+from tortoisehg.hgtk import thgmq, textview
 
 class BranchOperationDialog(gtk.Dialog):
     def __init__(self, branch, close, repo):
@@ -384,7 +387,7 @@ class GCommit(GStatus):
 
         mbox = gtk.HBox()
         self.connect('thg-accept', self.thgaccept)
-        self.branchbutton = gtk.Button()
+        self.branchbutton = gtk.Button(use_underline=False)
         self.branchbutton.connect('clicked', self.branch_clicked)
         mbox.pack_start(self.branchbutton, False, False, 2)
         if self.is_merge():
@@ -434,7 +437,7 @@ class GCommit(GStatus):
         self.add_accelerator('thg-reflow', accelgroup, key,
                         modifier, gtk.ACCEL_VISIBLE)
 
-        self.text = gtk.TextView()
+        self.text = textview.UndoableTextView(accelgroup=accelgroup)
         self.text.connect('populate-popup', self.msg_add_to_popup)
         self.connect('thg-reflow', self.thgreflow, self.text)
         self.text.modify_font(self.fonts['comment'])
@@ -450,6 +453,7 @@ class GCommit(GStatus):
             self.mqwidget = thgmq.MQWidget(self.repo, accelgroup,
                                            self.tooltips)
             self.mqwidget.connect('repo-invalidated', self.repo_invalidated)
+            self.mqwidget.connect('close-mq', lambda *a: self.enable_mqpanel(False))
 
             def wrapframe(widget):
                 frame = gtk.Frame()
@@ -655,7 +659,7 @@ class GCommit(GStatus):
 
         btn = self.get_toolbutton('commit')
         btn.set_label(label)
-        btn.set_tooltip(self.tooltips, tooltip)
+        self.tooltips.set_tip(btn, tooltip)
 
     def get_head_info(self):
         def ishead(ctx):
@@ -742,7 +746,7 @@ class GCommit(GStatus):
     def get_custom_menus(self):
         def commit(menuitem, files):
             if self.ready_message() and self.isuptodate():
-                def callback():
+                def callback(ret):
                     self.reload_status()
                     abs = [self.repo.wjoin(file) for file in files]
                     shlib.shell_notify(abs)
@@ -768,8 +772,18 @@ class GCommit(GStatus):
         self.branchbutton.set_label(text)
 
     def check_undo(self):
-        can_undo = os.path.exists(self.repo.sjoin("undo")) and \
-                self.last_commit_id is not None
+        can_undo = False
+        if os.path.exists(self.repo.sjoin('undo')):
+            can_undo = (self.last_commit_id is not None)
+            try:
+                # when hg-1.5 support is dropped, self.last_commit_id
+                # can be removed
+                args = self.repo.opener('undo.desc', 'r').read().splitlines()
+                if args[1] == 'commit':
+                    can_undo = True
+                    self.last_commit_id = self.get_tip_rev(True)
+            except (IOError, IndexError):
+                pass
         self.cmd_set_sensitive('undo', can_undo)
 
     def check_merge(self):
@@ -782,10 +796,12 @@ class GCommit(GStatus):
 
             # pre-fill commit message, if not modified
             buf = self.text.get_buffer()
-            if not buf.get_modified():
-                buf.set_text(_('Merge '))
-                buf.set_modified(False)
-
+            if buf.get_modified():
+                return
+            engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
+            msgset = keep._('Merge ')
+            buf.set_text(engmsg and msgset['id'] or msgset['str'])
+            buf.set_modified(False)
 
     def check_patch_queue(self):
         'See if an MQ patch is applied, switch to qrefresh mode'
@@ -824,7 +840,7 @@ class GCommit(GStatus):
                 revs = cmdutil.revrange(self.repo, ['tip'])
                 fp = cStringIO.StringIO()
                 opts = patch.diffopts(self.ui, self.opts)
-                patch.export(self.repo, revs, fp=fp, opts=opts)
+                hglib.export(self.repo, revs, fp=fp, opts=opts)
                 text = fp.getvalue().splitlines(True)
                 buf = self.diff_highlight_buffer(text)
                 self.patch_text.set_buffer(buf)
@@ -857,10 +873,13 @@ class GCommit(GStatus):
                     commitable += '?!'
             return self.relevant_checked_files(commitable)
 
-        def callback():
-            self.reload_status()
-            files = [self.repo.wjoin(x) for x in commit_list]
-            shlib.shell_notify(files)
+        def callback(ret):
+            if ret == 0 and self.repo.ui.configbool('tortoisehg', 'closeci'):
+                self.destroy()
+            else:
+                self.reload_status()
+                files = [self.repo.wjoin(x) for x in commit_list]
+                shlib.shell_notify(files)
 
         if self.qnew:
             commit_list = get_list()
@@ -902,6 +921,19 @@ class GCommit(GStatus):
                 gdialog.Prompt(_('Commit'),
                         _('Unable to create ') + backupdir, self).run()
                 return
+
+        def finish(ret):
+            os.chdir(cwd)
+            # restore backup files
+            try:
+                for realname, tmpname in backups.iteritems():
+                    util.copyfile(tmpname, repo.wjoin(realname))
+                    os.unlink(tmpname)
+                os.rmdir(backupdir)
+            except OSError:
+                pass
+            callback(ret)
+
         try:
             # backup continues
             allchunks = []
@@ -909,8 +941,8 @@ class GCommit(GStatus):
                 cf = util.pconvert(f)
                 if cf in self.subrepos: continue
                 if cf not in self.status[0]: continue
-                if f not in self.filechunks: continue
-                chunks = self.filechunks[f]
+                if f not in self.chunks: continue
+                chunks = self.chunks[f]
                 if len(chunks) < 2: continue
 
                 # unfiltered files do not go through backup-revert-patch cycle
@@ -954,18 +986,6 @@ class GCommit(GStatus):
                                 _('Unable to apply patch'), self).run()
                         return
 
-            def finish():
-                os.chdir(cwd)
-                # restore backup files
-                try:
-                    for realname, tmpname in backups.iteritems():
-                        util.copyfile(tmpname, repo.wjoin(realname))
-                        os.unlink(tmpname)
-                    os.rmdir(backupdir)
-                except OSError:
-                    pass
-                callback()
-
             # 4. We prepared working directory according to filtered patch.
             #    Now is the time to delegate the job to commit/qrefresh
             #    or the like!
@@ -981,7 +1001,7 @@ class GCommit(GStatus):
 
     def undo_clicked(self, toolbutton, data=None):
         response = gdialog.Confirm(_('Confirm Undo Commit'),
-                [], self, _('Undo last commit')).run()
+                [], self, _('Undo last commit?')).run()
         if response != gtk.RESPONSE_YES:
             return
 
@@ -1017,7 +1037,7 @@ class GCommit(GStatus):
         # prompts). Problem?
         self.opts['addremove'] = True
         if self.qnew or self.qheader is not None:
-            cmdline = ['hg', 'addremove', '--verbose']
+            cmdline = ['hg', 'addremove', '--verbose', '--']
             cmdline += [self.repo.wjoin(x) for x in files]
             self.execute_command(cmdline, force=True)
         return True
@@ -1084,9 +1104,8 @@ class GCommit(GStatus):
             # bring up the config dialog for user to enter their username.
             # But since we can't be sure they will do it right, we will
             # have them to retry, to re-trigger the checking mechanism.
-            dlg = thgconfig.ConfigDialog(False)
+            dlg = thgconfig.ConfigDialog(False, focus='ui.username')
             dlg.show_all()
-            dlg.focus_field('ui.username')
             dlg.run()
             dlg.hide()
             self.refreshui()
@@ -1107,20 +1126,29 @@ class GCommit(GStatus):
         if self.nextbranch:
             # response: 0=Yes, 1=No, 2=Cancel
             if self.nextbranch in self.repo.branchtags():
-                if self.nextbranch in [p.branch() for p in self.repo.parents()]:
-                    response = 0
+                pb = [p.branch() for p in self.repo.parents()]
+                if self.nextbranch in pb:
+                    resp = 0
                 else:
-                    response = gdialog.CustomPrompt(_('Confirm Override Branch'),
-                        _('A branch named "%s" already exists,\n'
-                        'override?') % self.nextbranch, self,
+                    rev = self.repo[self.nextbranch].rev()
+                    resp = gdialog.CustomPrompt(_('Confirm Branch Change'),
+                        _('Named branch "%s" already exists, '
+                          'last used in revision %d\n'
+                          'Yes\t- Make commit restarting this named branch\n'
+                          'No\t- Make commit without changing branch\n'
+                          'Cancel - Cancel this commit') % (self.nextbranch,
+                              rev), self,
                         (_('&Yes'), _('&No'), _('&Cancel')), 2, 2).run()
             else:
-                response = gdialog.CustomPrompt(_('Confirm New Branch'),
-                    _('Create new named branch "%s"?') % self.nextbranch,
+                resp = gdialog.CustomPrompt(_('Confirm New Branch'),
+                    _('Create new named branch "%s" with this commit?\n'
+                      'Yes\t- Start new branch with this commit\n'
+                      'No\t- Make commit without branch change\n'
+                      'Cancel - Cancel this commit') % self.nextbranch,
                     self, (_('&Yes'), _('&No'), _('&Cancel')), 2, 2).run()
-            if response == 0:
+            if resp == 0:
                 self.repo.dirstate.setbranch(self.nextbranch)
-            elif response == 2:
+            elif resp == 2:
                 return
         elif self.closebranch:
             cmdline.append('--close-branch')
@@ -1146,6 +1174,7 @@ class GCommit(GStatus):
         cmdline += ['--message', hglib.fromutf(self.opts['message'])]
         if self.qnew:
             cmdline += [hglib.fromutf(self.get_qnew_name())]
+        cmdline.append('--')
         cmdline += files
         if autopush:
             cmdline = (cmdline, ['hg', 'push'])
@@ -1154,7 +1183,7 @@ class GCommit(GStatus):
                 # refresh overlay icons and commit dialog
                 self.closebranch = False
                 self.nextbranch = None
-                self.filechunks = {} # force re-read of chunks
+                self.chunks.clear_filechunks()  # force re-read of chunks
                 buf = self.text.get_buffer()
                 if buf.get_modified():
                     self.update_recent_messages(self.opts['message'])
@@ -1165,23 +1194,26 @@ class GCommit(GStatus):
                     self.mode = 'commit'
                     self.qnew = False
                 elif self.qheader is None:
-                    self.text.set_buffer(gtk.TextBuffer())
+                    self.text.set_buffer(textview.UndoableTextBuffer())
                     self.msg_cbbox.set_active(-1)
                     self.last_commit_id = self.get_tip_rev(True)
                 if self.notify_func:
                     self.notify_func(*self.notify_args)
-                text = _('Finish committing')
+                if autopush:
+                    text = _('Finished committing and pushing')
+                else:
+                    text = _('Finished committing')
             elif return_code is None:
                 text = _('Aborted committing')
             else:
                 text = _('Failed to commit')
             self.stbar.set_idle_text(text)
-            callback()
+            callback(return_code)
         if not self.execute_command(cmdline, done,
                     status=_('Committing changes...'),
                     title=_('Commit')):
             gdialog.Prompt(_('Cannot run now'),
-                           _('Please try again after running '
+                           _('Please try again after the running '
                              'operation is completed'), self).run()
 
     def get_tip_rev(self, refresh=False):
@@ -1296,9 +1328,8 @@ class GCommit(GStatus):
         buf.set_text('\n'.join(lines))                       
 
     def msg_config(self, sender):
-        dlg = thgconfig.ConfigDialog(True)
+        dlg = thgconfig.ConfigDialog(True, focus='tortoisehg.summarylen')
         dlg.show_all()
-        dlg.focus_field('tortoisehg.summarylen')
         dlg.run()
         dlg.hide()
         self.refreshui()
