@@ -136,18 +136,31 @@ def PatchContext(repo, patchpath, cache={}):
 
 class patchctx(object):
 
-    def __init__(self, patchpath, repo):
+    def __init__(self, patchpath, repo, patchHandle=None):
+        """ Read patch context from file
+        :param patchHandle: If set, then the patch is a temporary.
+            The provided handle is used to read the patch and
+            the patchpath contains the name of the patch. 
+            The handle is NOT closed.
+        """
         self._path = patchpath
         self._patchname = os.path.basename(patchpath)
         self._repo = repo
-        pf = open(patchpath)
+        if patchHandle:
+            pf = patchHandle
+            pf_start_pos = pf.tell()
+        else:
+            pf = open(patchpath)
         try:
             data = patch.extract(self._repo.ui, pf)
             tmpfile, msg, user, date, branch, node, p1, p2 = data
             if tmpfile:
                 os.unlink(tmpfile)
         finally:
-            pf.close()
+            if patchHandle:
+                pf.seek(pf_start_pos)
+            else:
+                pf.close()
         if not msg and hasattr(repo, 'mq'):
             # attempt to get commit message
             from hgext import mq
@@ -296,7 +309,10 @@ class SummaryInfo(object):
                 extra = ctx.extra()
                 cvt = extra.get('convert_revision', '')
                 if cvt.startswith('svn:'):
-                    return cvt.split('/', 1)[-1]
+                    result = cvt.split('/', 1)[-1]
+                    if cvt != result:
+                        return result
+                    return cvt.split('@')[-1]
                 else:
                     return None
             elif item == 'ishead':
@@ -350,10 +366,10 @@ class SummaryInfo(object):
             elif item in ('revnum', 'p4', 'svn'):
                 return str(value)
             elif item in ('rawbranch', 'branch'):
-                return gtklib.markup(' %s ' % value, color='black',
+                return gtklib.markup(' %s ' % value, color=gtklib.BLACK,
                                      background=gtklib.PGREEN)
             elif item in ('rawtags', 'tags'):
-                opts = dict(color='black', background=gtklib.PYELLOW)
+                opts = dict(color=gtklib.BLACK, background=gtklib.PYELLOW)
                 tags = [gtklib.markup(' %s ' % tag, **opts) for tag in value]
                 return ' '.join(tags)
             elif item in ('desc', 'summary', 'user', 'shortuser',
@@ -470,6 +486,13 @@ class SummaryBase(object):
         return self.info.get_widget(item, self, self.ctx, self.custom, **kargs)
 
     def update(self, target=None, custom=None, repo=None):
+        self.ctx = None
+        if type(target) == patchctx:
+            # If a patchctx is specified as target, use it instead
+            # of creating a context from revision or patch file
+            self.ctx = target
+            target = None
+            self.target = None
         if target is None:
             target = self.target
         if target is not None:
@@ -481,7 +504,8 @@ class SummaryBase(object):
             repo = self.repo
         if repo is not None:
             self.repo = repo
-        self.ctx = create_context(repo, target)
+        if self.ctx is None:
+            self.ctx = create_context(repo, target)
         if self.ctx is None:
             return False # cannot update
         return True
