@@ -14,26 +14,13 @@ import cStringIO
 import Queue
 
 from mercurial import hg, ui, mdiff, cmdutil, match, util, error
+from mercurial import similar
 
 from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib, shlib, paths, thread2, settings
 
 from tortoisehg.hgtk import gtklib, statusbar
 
-# This function and some key bits below borrowed ruthelessly from
-# Peter Arrenbrecht <peter.arrenbrecht@gmail.com>
-# Thanks!
-def findmoves(repo, added, removed, threshold):
-    '''find renamed files -- yields (before, after, score) tuples'''
-    ctx = repo['.']
-    for r in removed:
-        rr = ctx.filectx(r).data()
-        for a in added:
-            aa = repo.wread(a)
-            if aa == rr:
-                yield r, a, 1.0
-                break
-
 class DetectRenameDialog(gtk.Window):
     'Detect renames after they occur'
     def __init__(self):
@@ -196,7 +183,7 @@ class DetectRenameDialog(gtk.Window):
         self.buf = gtk.TextBuffer()
         self.buf.create_tag('removed', foreground=gtklib.DRED)
         self.buf.create_tag('added', foreground=gtklib.DGREEN)
-        self.buf.create_tag('position', foreground='#FF8000')
+        self.buf.create_tag('position', foreground=gtklib.DORANGE)
         self.buf.create_tag('header', foreground=gtklib.DBLUE)
         diffview = gtk.TextView(self.buf)
         scroller.add(diffview)
@@ -259,7 +246,7 @@ class DetectRenameDialog(gtk.Window):
         self.settings.set_value('dims', (rect.width, rect.height))
         self.settings.write()
 
-    def find_renames(self):
+    def find_renames(self, copy=False):
         'User pressed "find renames" button'
         canmodel = self.cantree.get_model()
         canmodel.clear()
@@ -268,13 +255,17 @@ class DetectRenameDialog(gtk.Window):
             return
         tgts = [ umodel[p][0] for p in upaths ]
         q = Queue.Queue()
-        thread = thread2.Thread(target=self.search_thread, args=(q, tgts))
+        thread = thread2.Thread(target=self.search_thread,
+                                args=(q, tgts, copy))
         thread.start()
         self.stbar.begin()
-        self.stbar.set_text(_('finding source of ') + ', '.join(tgts))
+        text = _('finding source of ') + ', '.join(tgts)
+        if len(text) > 60:
+            text = text[:60]+'...'
+        self.stbar.set_text(text)
         gobject.timeout_add(50, self.search_wait, thread, q)
 
-    def search_thread(self, q, tgts):
+    def search_thread(self, q, tgts, copy):
         hglib.invalidaterepo(self.repo)
         srcs = []
         audit_path = util.path_auditor(self.repo.root)
@@ -290,26 +281,27 @@ class DetectRenameDialog(gtk.Window):
             if (not good or not util.lexists(target)
                 or (os.path.isdir(target) and not os.path.islink(target))):
                 srcs.append(abs)
-            elif not self.adjustment and status == 'n':
+            elif copy and status == 'n':
                 # looking for copies, so any revisioned file is a
                 # potential source (yes, this will be expensive)
                 # Added and removed files are not considered as copy
                 # sources.
                 srcs.append(abs)
-        if self.adjustment:
-            simularity = self.adjustment.get_value() / 100.0;
-            gen = cmdutil.findrenames
-        else:
+        if copy:
             simularity = 1.0
-            gen = findmoves
-        for old, new, score in gen(self.repo, tgts, srcs, simularity):
+        else:
+            simularity = self.adjustment.get_value() / 100.0;
+
+        gen = similar.findrenames(self.repo, tgts, srcs, simularity)
+        for old, new, score in gen:
             q.put( [old, new, '%d%%' % (score*100)] )
 
     def search_wait(self, thread, q):
         canmodel = self.cantree.get_model()
-        while q.qsize():
+        while canmodel and q.qsize():
             source, dest, sim = q.get(0)
-            canmodel.append( [source, hglib.toutf(source), dest, hglib.toutf(dest), sim, True] )
+            canmodel.append( [source, hglib.toutf(source), dest,
+                              hglib.toutf(dest), sim, True] )
         if thread.isAlive():
             return True
         else:
@@ -319,11 +311,12 @@ class DetectRenameDialog(gtk.Window):
     def find_copies(self):
         'User pressed "find copies" button'
         # call rename function with simularity = 100%
-        self.find_renames()
+        self.find_renames(copy=True)
 
     def accept_match(self):
         'User pressed "accept match" button'
         hglib.invalidaterepo(self.repo)
+        wctx = self.repo[None]
         canmodel, upaths = self.cantree.get_selection().get_selected_rows()
         for path in upaths:
             row = canmodel[path]
@@ -332,8 +325,8 @@ class DetectRenameDialog(gtk.Window):
                 continue
             if not os.path.exists(self.repo.wjoin(src)):
                 # Mark missing rename source as removed
-                self.repo.remove([src])
-            self.repo.copy(src, dest)
+                wctx.remove([src])
+            wctx.copy(src, dest)
             shlib.shell_notify([self.repo.wjoin(src), self.repo.wjoin(dest)])
             if self.notify_func:
                 self.notify_func()
