@@ -16,7 +16,7 @@ from mercurial import hg, ui, extensions, error
 from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib, settings
 
-from tortoisehg.hgtk import gtklib, dialog, thgconfig, hgcmd
+from tortoisehg.hgtk import gtklib, dialog, thgconfig, hgcmd, textview
 
 class EmailDialog(gtk.Window):
     """ Send patches or bundles via email """
@@ -30,7 +30,7 @@ class EmailDialog(gtk.Window):
         self.revargs = revargs
 
         self.tbar = gtk.Toolbar()
-        self.tips = gtk.Tooltips()
+        self.tips = gtklib.Tooltips()
 
         tbuttons = [
                 self._toolbutton(gtk.STOCK_GOTO_LAST, _('Send'),
@@ -95,10 +95,9 @@ class EmailDialog(gtk.Window):
         table = gtklib.LayoutTable()
         flagframe.add(table)
 
-        self.tooltips = gtk.Tooltips()
         self._normal = gtk.RadioButton(None, _('Send changesets as Hg patches'))
         table.add_row(self._normal)
-        self.tooltips.set_tip(self._normal,
+        self.tips.set_tip(self._normal,
                 _('Hg patches (as generated by export command) are compatible'
                 ' with most patch programs.  They include a header which'
                 ' contains the most important changeset metadata.'))
@@ -106,7 +105,7 @@ class EmailDialog(gtk.Window):
         self._git = gtk.RadioButton(self._normal,
                 _('Use extended (git) patch format'))
         table.add_row(self._git)
-        self.tooltips.set_tip(self._git,
+        self.tips.set_tip(self._git,
                 _('Git patches can describe binary files, copies, and'
                 ' permission changes, but recipients may not be able to'
                 ' use them if they are not using git or Mercurial.'))
@@ -114,7 +113,7 @@ class EmailDialog(gtk.Window):
         self._plain = gtk.RadioButton(self._normal,
                 _('Plain, do not prepend Hg header'))
         table.add_row(self._plain)
-        self.tooltips.set_tip(self._plain,
+        self.tips.set_tip(self._plain,
                 _('Stripping Mercurial header removes username and parent'
                 ' information.  Only useful if recipient is not using'
                 ' Mercurial (and does not like to see the headers).'))
@@ -123,24 +122,24 @@ class EmailDialog(gtk.Window):
                 _('Send single binary bundle, not patches'))
         table.add_row(self._bundle)
         if revargs[0] in ('--outgoing', '-o'):
-            self.tooltips.set_tip(self._bundle,
+            self.tips.set_tip(self._bundle,
                 _('Bundles store complete changesets in binary form.'
                 ' Upstream users can pull from them. This is the safest'
                 ' way to send changes to recipient Mercurial users.'))
         else:
             self._bundle.set_sensitive(False)
-            self.tooltips.set_tip(self._bundle,
+            self.tips.set_tip(self._bundle,
                 _('This feature is only available when sending outgoing'
                 ' changesets. It is not applicable with revision ranges.'))
 
         self._attach = gtk.CheckButton(_('attach'))
-        self.tooltips.set_tip(self._attach,
+        self.tips.set_tip(self._attach,
                 _('send patches as attachments'))
         self._inline = gtk.CheckButton(_('inline'))
-        self.tooltips.set_tip(self._inline,
+        self.tips.set_tip(self._inline,
                 _('send patches as inline attachments'))
         self._diffstat = gtk.CheckButton(_('diffstat'))
-        self.tooltips.set_tip(self._diffstat,
+        self.tips.set_tip(self._diffstat,
                 _('add diffstat output to messages'))
         table.add_row(self._attach, self._inline, self._diffstat, padding=False)
 
@@ -168,7 +167,9 @@ class EmailDialog(gtk.Window):
         vbox.pack_start(hbox, False, False, 4)
 
         # Description TextView
-        self.descview = gtk.TextView(buffer=None)
+        accelgroup = gtk.AccelGroup()
+        self.add_accel_group(accelgroup)
+        self.descview = textview.UndoableTextView(accelgroup=accelgroup)
         self.descview.set_editable(True)
         fontcomment = hglib.getfontconfig()['fontcomment']
         self.descview.modify_font(pango.FontDescription(fontcomment))
@@ -191,7 +192,7 @@ class EmailDialog(gtk.Window):
     def _toolbutton(self, stock, label, handler, tip):
         tbutton = gtk.ToolButton(stock)
         tbutton.set_label(label)
-        tbutton.set_tooltip(self.tips, tip)
+        self.tips.set_tip(tbutton, tip)
         tbutton.connect('clicked', handler)
         return tbutton
 
@@ -211,13 +212,23 @@ class EmailDialog(gtk.Window):
             self.repo = None
             return
 
+        def getfromaddr(ui):
+            """Get sender address in the same manner as patchbomb"""
+            addr = ui.config('email', 'from') or ui.config('patchbomb', 'from')
+            if addr:
+                return addr
+            try:
+                return repo.ui.username()
+            except error.Abort:
+                return ''
+
         if initial:
             # Only zap these fields at startup
             self._tobox.child.set_text(hglib.fromutf(repo.ui.config('email', 'to', '')))
             self._ccbox.child.set_text(hglib.fromutf(repo.ui.config('email', 'cc', '')))
-            self._frombox.child.set_text(hglib.fromutf(repo.ui.config('email', 'from', '')))
+            self._frombox.child.set_text(hglib.fromutf(getfromaddr(repo.ui)))
             self._subjbox.child.set_text(hglib.fromutf(repo.ui.config('email', 'subject', '')))
-            self.tooltips.set_tip(self._eventbox,
+            self.tips.set_tip(self._eventbox,
                     _('Patch series description is sent in initial summary'
                     ' email with [PATCH 0 of N] subject.  It should describe'
                     ' the effects of the entire patch series.  When emailing'
@@ -235,9 +246,8 @@ class EmailDialog(gtk.Window):
             self._flaglist.append(['STABLE'])
 
     def _on_conf_clicked(self, button):
-        dlg = thgconfig.ConfigDialog(False)
+        dlg = thgconfig.ConfigDialog(False, focus='email.from')
         dlg.show_all()
-        dlg.focus_field('email.from')
         dlg.run()
         dlg.hide()
         self._refresh(False)
@@ -281,9 +291,8 @@ class EmailDialog(gtk.Window):
             if not self.repo.ui.config('smtp', 'host'):
                 dialog.info_dialog(self, _('Info required'),
                             _('You must configure SMTP'))
-                dlg = thgconfig.ConfigDialog(False)
+                dlg = thgconfig.ConfigDialog(False, focus='smtp.host')
                 dlg.show_all()
-                dlg.focus_field('smtp.host')
                 dlg.run()
                 dlg.hide()
                 self._refresh(False)
@@ -328,10 +337,12 @@ class EmailDialog(gtk.Window):
             cmdline += ['--intro']
         tmpfile = None
         try:
-            fd, tmpfile = tempfile.mkstemp(prefix="thg_emaildesc_")
-            os.write(fd, desc)
-            os.close(fd)
-            cmdline += ['--desc', tmpfile]
+            if desc or not hasattr(extensions.find('patchbomb'), 'introneeded'):
+                # --desc is interpreted differently after hg 1.5
+                fd, tmpfile = tempfile.mkstemp(prefix="thg_emaildesc_")
+                os.write(fd, desc)
+                os.close(fd)
+                cmdline += ['--desc', tmpfile]
             cmdline.extend(self.revargs)
 
             dlg = hgcmd.CmdDialog(cmdline)
