@@ -25,30 +25,37 @@ class GtkUi(ui.ui):
     main thread to pickup.
     '''
     def __init__(self, src=None, outputq=None, errorq=None, dialogq=None,
-            responseq=None):
+            responseq=None, progressq=None):
         super(GtkUi, self).__init__(src)
         if src:
             self.outputq = src.outputq
             self.errorq = src.errorq
             self.dialogq = src.dialogq
             self.responseq = src.responseq
+            self.progressq = src.progressq
         else:
             self.outputq = outputq
             self.errorq = errorq
             self.dialogq = dialogq
             self.responseq = responseq
+            self.progressq = progressq
         self.setconfig('ui', 'interactive', 'on')
         self.setconfig('progress', 'disable', 'True')
 
-    def write(self, *args):
-        if hglib.uiwrite(self, args):
+    def write(self, *args, **opts):
+        if self._buffers:
+            self._buffers[-1].extend([str(a) for a in args])
+        else:
             for a in args:
-                self.outputq.put(str(a))
+                self.outputq.put((str(a), opts.get('label', '')))
 
-    def write_err(self, *args):
+    def write_err(self, *args, **opts):
         for a in args:
             self.errorq.put(str(a))
 
+    def label(self, msg, label):
+        return msg
+
     def flush(self):
         pass
 
@@ -90,6 +97,9 @@ class GtkUi(ui.ui):
             raise util.Abort(_('response expected'))
         return r
 
+    def progress(self, topic, pos, item='', unit='', total=None):
+        self.progressq.put( (topic, item, pos, total, unit) )
+
 
 class HgThread(thread2.Thread):
     '''
@@ -103,8 +113,9 @@ class HgThread(thread2.Thread):
         self.errorq = Queue.Queue()
         self.dialogq = Queue.Queue()
         self.responseq = Queue.Queue()
+        self.progressq = Queue.Queue()
         self.ui = GtkUi(None, self.outputq, self.errorq, self.dialogq,
-                        self.responseq)
+                        self.responseq, self.progressq)
         self.args = args
         self.ret = None
         self.postfunc = postfunc
@@ -117,6 +128,9 @@ class HgThread(thread2.Thread):
     def geterrqueue(self):
         return self.errorq
 
+    def getprogqueue(self):
+        return self.progressq
+
     def return_code(self):
         '''
         None - command is incomplete, possibly exited with exception
@@ -159,12 +173,13 @@ class HgThread(thread2.Thread):
         try:
             for k, v in self.ui.configitems('defaults'):
                 self.ui.setconfig('defaults', k, '')
+            l = 'control'
             ret = hglib.dispatch._dispatch(self.ui, self.args)
             if ret:
-                self.ui.write(_('[command returned code %d ') % int(ret))
+                self.ui.write(_('[command returned code %d ') % int(ret), label=l)
             else:
-                self.ui.write(_('[command completed successfully '))
-            self.ui.write(time.asctime() + ']\n')
+                self.ui.write(_('[command completed successfully '), label=l)
+            self.ui.write(time.asctime() + ']\n', label=l)
             self.ret = ret or 0
             if self.postfunc:
                 self.postfunc(ret)
