@@ -194,7 +194,8 @@ class TreeView(gtk.ScrolledWindow):
                 self.grapher = revision_grapher(self.repo, start, end,
                         only_branch, noheads, self.opts.get('branch-color'))
         elif opts.get('revlist', None):
-            self.grapher = dumb_log_generator(self.repo, opts['revlist'])
+            graphcol = True
+            self.grapher = revision_grapher(self.repo, only_revs=opts['revlist'])
         else:
             self.grapher = filtered_log_generator(self.repo, pats, opts)
         self.opts['show-graph'] = graphcol
@@ -354,18 +355,38 @@ class TreeView(gtk.ScrolledWindow):
             self.treeview.set_cursor(row)
             self.treeview.grab_focus()
         elif load:
-            handler = None
+            handlers = [None, None]
 
             def loaded(dummy):
                 if revid in self.index:
-                    if handler is not None:
-                        self.disconnect(handler)
+                    hload = handlers[0]
+                    if hload is not None:
+                        self.disconnect(hload)
+                        handlers[0] = None
                     self.set_revision_id(revid)
                     self.scroll_to_revision(revid)
                 else:
                     self.next_revision_batch(self.batchsize)
+            def stopped(dummy):
+                hload, hstop = handlers
+                if hload is not None:
+                    self.disconnect(hload)
+                    handlers[0] = None
+                if hstop is not None:
+                    self.disconnect(hstop)
+                    handlers[1] = None
+                self.stbar.set_text(_('Changeset not found in current view'))
 
-            handler = self.connect('batch-loaded', loaded)
+            try:
+                ctx = self.repo[revid]
+                if ctx.rev() == -1:
+                    self.stbar.set_text(_('Null changeset is not viewable'))
+                    return
+            except Exception, e:
+                self.stbar.set_text(str(e))
+                return
+            handlers[0] = self.connect('batch-loaded', loaded)
+            handlers[1] = self.connect('revisions-loaded', stopped)
             self.next_revision_batch(self.batchsize)
 
     def refresh(self, graphcol, pats, opts):
