@@ -37,12 +37,6 @@ class SynchDialog(gtk.Window):
         self.last_drop_time = None
         self.lastcmd = []
 
-        # Replace stdout file descriptor with our own pipe
-        self.oldstdout = os.dup(sys.__stdout__.fileno())
-        self.stdoutq = Queue.Queue()
-        self.readfd, writefd = os.pipe()
-        os.dup2(writefd, sys.__stdout__.fileno())
-
         # persistent app data
         self._settings = settings.Settings('synch')
         self.set_default_size(655, 552)
@@ -54,7 +48,7 @@ class SynchDialog(gtk.Window):
 
         # toolbar
         self.tbar = gtk.Toolbar()
-        self.tips = gtk.Tooltips()
+        self.tips = gtklib.Tooltips()
         self.stop_button = self.toolbutton(gtk.STOCK_STOP,
                 _('Stop'), self.stop_clicked, tip=_('Stop the hg operation'))
         self.stop_button.set_sensitive(False)
@@ -180,12 +174,15 @@ class SynchDialog(gtk.Window):
         self.force = gtk.CheckButton(_('Force pull or push'))
         self.tips.set_tip(self.force, _('Run even when remote repository'
                 ' is unrelated.'))
+        self.newbranch = gtk.CheckButton(_('Push new branch'))
+        self.tips.set_tip(self.newbranch, _('Allow pushing a new branch'))
         self.use_proxy = gtk.CheckButton(_('Use proxy server'))
         if ui.ui().config('http_proxy', 'host', ''):
             self.use_proxy.set_active(True)
         else:
             self.use_proxy.set_sensitive(False)
         chkopthbox.pack_start(self.force, False, False, 4)
+        chkopthbox.pack_start(self.newbranch, False, False, 4)
         chkopthbox.pack_start(self.use_proxy, False, False, 4)
 
         ## target revision option
@@ -245,8 +242,9 @@ class SynchDialog(gtk.Window):
         scrolledwindow.add(self.textview)
         self.textview.connect('populate-popup', self.add_to_popup)
         self.textbuffer = self.textview.get_buffer()
-        self.textbuffer.create_tag('error', weight=pango.WEIGHT_HEAVY,
-                                   foreground=gtklib.DRED)
+        gtklib.configstyles(self.repo.ui)
+        for tag, argdict in gtklib.TextBufferTags.iteritems():
+            self.textbuffer.create_tag(tag, **argdict)
         basevbox.pack_start(scrolledwindow, True, True)
 
         # statusbar
@@ -280,8 +278,15 @@ class SynchDialog(gtk.Window):
                     self.stdoutq.put(o)
                 else:
                     break
-        thread = threading.Thread(target=pollstdout, args=[])
-        thread.start()
+        self.stdoutq = Queue.Queue()
+        if os.name == 'nt':
+            # Only capture stdout on Windows. See issue #783, #1316.
+            # Replace stdout file descriptor with our own pipe
+            self.readfd, writefd = os.pipe()
+            self.oldstdout = os.dup(sys.__stdout__.fileno())
+            os.dup2(writefd, sys.__stdout__.fileno())
+            thread = threading.Thread(target=pollstdout, args=[])
+            thread.start()
 
     def update_pull_setting(self):
         ppull = self.repo.ui.config('tortoisehg', 'postpull', 'none')
@@ -397,8 +402,9 @@ class SynchDialog(gtk.Window):
         else:
             self.update_settings()
             self._settings.write()
-            os.dup2(self.oldstdout, sys.__stdout__.fileno())
-            os.close(self.oldstdout)
+            if os.name == 'nt':
+                os.dup2(self.oldstdout, sys.__stdout__.fileno())
+                os.close(self.oldstdout)
             return False
 
     def delete(self, widget, event):
@@ -415,7 +421,7 @@ class SynchDialog(gtk.Window):
 
         tbutton.set_label(label)
         if tip:
-            tbutton.set_tooltip(self.tips, tip)
+            self.tips.set_tip(tbutton, tip)
         tbutton.connect('clicked', handler, userdata)
         return tbutton
 
@@ -427,6 +433,8 @@ class SynchDialog(gtk.Window):
             opts['no-merges'] = ['--no-merges']
         if self.force.get_active():
             opts['force'] = ['--force']
+        if self.newbranch.get_active():
+            opts['newbranch'] = ['--new-branch']
         if self.newestfirst.get_active():
             opts['newest-first'] = ['--newest-first']
         remotecmd = self.cmdentry.get_text().strip()
@@ -464,6 +472,7 @@ class SynchDialog(gtk.Window):
         cmd = ['push']
         cmd += aopts.get('rev', [])
         cmd += aopts.get('force', [])
+        cmd += aopts.get('newbranch', [])
         cmd += aopts.get('remotecmd', [])
         self.exec_cmd(cmd)
 
@@ -538,7 +547,7 @@ class SynchDialog(gtk.Window):
     def exec_cmd(self, cmd):
         if self.cmd_running():
             dialog.error_dialog(self, _('Cannot run now'),
-                _('Please try again after the previous command is completed'))
+                _('Please try again after the previous command has completed'))
             return
 
         self.stop_button.set_sensitive(True)
@@ -553,7 +562,7 @@ class SynchDialog(gtk.Window):
         cmdline += ['--verbose']
         if proxy_host and not use_proxy:
             cmdline += ["--config", "http_proxy.host="]
-        cmdline += [remote_path]
+        cmdline += ['--', remote_path]
         self.lastcmd = cmdline
 
         # show command to be executed
@@ -592,8 +601,9 @@ class SynchDialog(gtk.Window):
             self.textbuffer.set_text(msg)
 
     def write_err(self, msg):
+        tags = gtklib.gettags('ui.error')
         enditer = self.textbuffer.get_end_iter()
-        self.textbuffer.insert_with_tags_by_name(enditer, msg, 'error')
+        self.textbuffer.insert_with_tags_by_name(enditer, msg, *tags)
         self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
 
     def process_queue(self):
@@ -601,10 +611,17 @@ class SynchDialog(gtk.Window):
         Handle all the messages currently in the queue (if any).
         """
         self.hgthread.process_dialogs()
+        enditer = self.textbuffer.get_end_iter()
         while self.hgthread.getqueue().qsize():
             try:
-                msg = self.hgthread.getqueue().get(0)
-                self.write(msg)
+                msg, label = self.hgthread.getqueue().get(0)
+                msg = hglib.toutf(msg)
+                tags = gtklib.gettags(label)
+                if tags:
+                    self.textbuffer.insert_with_tags_by_name(enditer, msg, *tags)
+                else:
+                    self.textbuffer.insert(enditer, msg)
+                self.textview.scroll_to_mark(self.textbuffer.get_insert(), 0)
             except Queue.Empty:
                 pass
         while self.hgthread.geterrqueue().qsize():
@@ -644,6 +661,7 @@ class SynchDialog(gtk.Window):
         'reventry.text': '',
         'cmdentry.text': '',
         'force.active': False,
+        'newbranch.active': False,
         'showpatch.active': False,
         'newestfirst.active': False,
         'nomerge.active': False,}
