@@ -11,7 +11,7 @@ import gtk.keysyms
 import gobject
 import pango
 
-from mercurial import error
+from mercurial import error, util
 
 from tortoisehg.util.i18n import _
 from tortoisehg.util import hglib
@@ -85,6 +85,9 @@ class MQWidget(gtk.VBox):
                           gobject.TYPE_NONE,
                           (object, # list of dropped files/dirs
                            str))   # raw string data
+        ,'close-mq': (gobject.SIGNAL_RUN_FIRST,
+                      gobject.TYPE_NONE,
+                      ())
     }
 
     def __init__(self, repo, accelgroup=None, tooltips=None):
@@ -131,6 +134,10 @@ class MQWidget(gtk.VBox):
         gtklib.idle_add_single_call(after_init)
         self.pack_start(tbar, False, False)
 
+        closebtn = tbar.append_button(gtk.STOCK_CLOSE, _('Close'))
+        closebtn.connect('clicked', lambda *a: self.emit('close-mq'))
+        self.btn['close'] = closebtn
+
         # center pane
         mainbox = gtk.VBox()
         self.pack_start(mainbox, True, True)
@@ -221,17 +228,21 @@ class MQWidget(gtk.VBox):
                     self.qrename_ui(model[paths[0]][MQ_NAME])
             self.list.connect('thg-rename', thgrename)
 
-            mod = gtk.gdk.CONTROL_MASK
-            def add(name, key, func, *args):
+            def add(name, key, mod, func, *args):
                 self.list.add_accelerator(name, accelgroup, key, mod, 0)
                 self.list.connect(name, lambda *a: func(*args))
-            add('mq-move-top', gtk.keysyms.Page_Up, self.qreorder_ui, MOVE_TOP)
-            add('mq-move-up', gtk.keysyms.Up, self.qreorder_ui, MOVE_UP)
-            add('mq-move-down', gtk.keysyms.Down, self.qreorder_ui, MOVE_DOWN)
-            add('mq-move-bottom', gtk.keysyms.Page_Down, self.qreorder_ui,
-                MOVE_BOTTOM)
-            add('mq-pop', gtk.keysyms.Left, self.qpop)
-            add('mq-push', gtk.keysyms.Right, self.qpush)
+            mod = gtk.gdk.CONTROL_MASK|gtk.gdk.SHIFT_MASK
+            add('mq-move-top', gtk.keysyms.Page_Up, mod,
+                self.qreorder_ui, MOVE_TOP)
+            add('mq-move-up', gtk.keysyms.Up, mod,
+                self.qreorder_ui, MOVE_UP)
+            add('mq-move-down', gtk.keysyms.Down, mod,
+                self.qreorder_ui, MOVE_DOWN)
+            add('mq-move-bottom', gtk.keysyms.Page_Down, mod,
+                self.qreorder_ui, MOVE_BOTTOM)
+            mod = gtk.gdk.CONTROL_MASK
+            add('mq-pop', gtk.keysyms.Up, mod, self.qpop)
+            add('mq-push', gtk.keysyms.Down, mod, self.qpush)
 
     ### public functions ###
 
@@ -298,7 +309,7 @@ class MQWidget(gtk.VBox):
         """
         if not self.is_operable():
             return
-        cmdline = ['hg', 'qgoto', patch]
+        cmdline = ['hg', 'qgoto', '--', patch]
         self.cmd.execute(cmdline, self.cmd_done)
 
     def qpop(self, all=False):
@@ -358,9 +369,11 @@ class MQWidget(gtk.VBox):
                 keep = True
             else:
                 return
-        cmdline = ['hg', 'qdelete'] + unapplied
+        cmdline = ['hg', 'qdelete']
         if keep:
             cmdline.append('--keep')
+        cmdline.append('--')
+        cmdline.extend(unapplied)
         self.cmd.execute(cmdline, self.cmd_done, noemit=True)
 
     def qrename(self, name, patch='qtip'):
@@ -374,7 +387,7 @@ class MQWidget(gtk.VBox):
         """
         if not name or not self.has_patch():
             return
-        cmdline = ['hg', 'qrename', patch, name]
+        cmdline = ['hg', 'qrename', '--', patch, name]
         self.cmd.execute(cmdline, self.cmd_done)
 
     def qrename_ui(self, patch='qtip'):
@@ -448,7 +461,7 @@ class MQWidget(gtk.VBox):
                       " into the current patch '%(qtip)s'?") % data).run()
         if ret != gtk.RESPONSE_YES:
             return
-        cmdline = ['hg', 'qfold'] + unapplied
+        cmdline = ['hg', 'qfold', '--'] + unapplied
         self.cmd.execute(cmdline, self.cmd_done)
 
     def qreorder(self, patch, op):
@@ -476,12 +489,14 @@ class MQWidget(gtk.VBox):
                 if maxidx < idx:
                     maxidx = idx
 
-        # find index of first unapplied patch in TreeView
+        # find index of first unapplied patch after last applied one in TreeView
+        uminidx = None
         for i, row in enumerate(model):
-            if self.is_unapplied(row[MQ_NAME]):
+            if uminidx is None and self.is_unapplied(row[MQ_NAME]):
                 uminidx = i
-                break
-        else:
+            elif self.is_applied(row[MQ_NAME]):
+                uminidx = None
+        if uminidx is None:
             return False
 
         # check whether operation is possible
@@ -546,6 +561,8 @@ class MQWidget(gtk.VBox):
         series = q.full_series[:]
         for pos, qpos in zip(dirty, qdirty):
             q.full_series[qpos] = series[model[pos][MQ_INDEX]]
+        if len(set(q.full_series)) != len(q.full_series):  # found duplicates
+            raise util.Abort(_('series become inconsistent during reorder'))
         q.series_dirty = True
         q.save_dirty()
 
@@ -721,11 +738,11 @@ class MQWidget(gtk.VBox):
 
         stat = row[MQ_STATUS]
         if stat == 'A':
-            cell.set_property('foreground', 'blue')
+            cell.set_property('foreground', gtklib.BLUE)
         elif stat == 'U':
-            cell.set_property('foreground', '#909090')
+            cell.set_property('foreground', gtklib.GREY)
         else:
-            cell.set_property('foreground', 'black')
+            cell.set_property('foreground', gtklib.NORMAL)
 
         patchname = row[MQ_NAME]
         if self.is_qtip(patchname):
@@ -938,10 +955,12 @@ class MQWidget(gtk.VBox):
         if row[MQ_INDEX] < 0:
             return
         patchname = row[MQ_NAME]
-        try:
-            ctx = self.repo[patchname]
-            revid = ctx.rev()
-        except (error.RepoError, error.RepoLookupError, error.LookupError):
+        q = self.repo.mq
+        q.parse_series()
+        applied = q.isapplied(patchname)
+        if applied:
+            revid = self.repo[applied[1]].rev()
+        else:
             revid = -1
         self.emit('patch-selected', revid, patchname)
 
