@@ -11,13 +11,16 @@ import shlex
 import time
 import inspect
 
-from mercurial import ui, util, extensions, match, bundlerepo, url
+from mercurial import ui, util, extensions, match, bundlerepo, url, cmdutil
 from mercurial import dispatch, encoding, templatefilters, filemerge
 
 _encoding = encoding.encoding
 _encodingmode = encoding.encodingmode
 _fallbackencoding = encoding.fallbackencoding
 
+# extensions which can cause problem with TortoiseHg
+_extensions_blacklist = ('color', 'pager', 'progress')
+
 from tortoisehg.util import paths
 from tortoisehg.util.i18n import _
 from tortoisehg.util.hgversion import hgversion
@@ -107,11 +110,29 @@ def getlivebranch(repo):
     '''return a list of live branch names in UTF-8'''
     lives = []
     deads = getdeadbranch(repo.ui)
-    for branch in repo.branchtags().keys(): # encoded in UTF-8
-        if branch not in deads:
-            lives.append(branch.replace('\0', ''))
+    cl = repo.changelog
+    for branch, heads in repo.branchmap().iteritems():
+        # branch encoded in UTF-8
+        if branch in deads:
+            # ignore branch names in tortoisehg.deadbranch
+            continue
+        bheads = [h for h in heads if ('close' not in cl.read(h)[5])]
+        if not bheads:
+            # ignore branches with all heads closed
+            continue
+        lives.append(branch.replace('\0', ''))
     return lives
 
+def getlivebheads(repo):
+    '''return a list of revs of live branch heads'''
+    bheads = []
+    for b, ls in repo.branchmap().iteritems():
+        bheads += [repo[x] for x in ls]
+    heads = [x.rev() for x in bheads if not x.extra().get('close')]
+    heads.sort()
+    heads.reverse()
+    return heads
+
 _hidetags = None
 def gethidetags(ui):
     global _hidetags
@@ -154,16 +175,6 @@ def getfontconfig(_ui=None):
                 _fontconfig[name] = val
     return _fontconfig
 
-def uiwrite(u, args):
-    '''
-    write args if there are buffers
-    returns True if the caller shall handle writing
-    '''
-    if u._buffers:
-        ui.ui.write(u, *args)
-        return False
-    return True
-
 def invalidaterepo(repo):
     repo.dirstate.invalidate()
     if isinstance(repo, bundlerepo.bundlerepository):
@@ -198,6 +209,21 @@ def loadextension(ui, name):
     if uisetup:
         uisetup(ui)
 
+def _loadextensionwithblacklist(orig, ui, name, path):
+    if name.startswith('hgext.') or name.startswith('hgext/'):
+        shortname = name[6:]
+    else:
+        shortname = name
+    if shortname in _extensions_blacklist and not path:  # only bundled ext
+        return
+
+    return orig(ui, name, path)
+
+def wrapextensionsloader():
+    """Wrap extensions.load(ui, name) for blacklist to take effect"""
+    extensions.wrapfunction(extensions, 'load',
+                            _loadextensionwithblacklist)
+
 def canonpaths(list):
     'Get canonical paths (relative to root) for list of files'
     # This is a horrible hack.  Please remove this when HG acquires a
@@ -312,7 +338,7 @@ def difftools(ui):
     return tools
 
 
-def hgcmd_toq(q, *args):
+def hgcmd_toq(q, label, args):
     '''
     Run an hg command in a background thread, pipe all output to a Queue
     object.  Assumes command is completely noninteractive.
@@ -322,14 +348,26 @@ def hgcmd_toq(q, *args):
             super(Qui, self).__init__(src)
             self.setconfig('ui', 'interactive', 'off')
 
-        def write(self, *args):
-            if uiwrite(self, args):
+        def write(self, *args, **opts):
+            if self._buffers:
+                self._buffers[-1].extend([str(a) for a in args])
+            else:
                 for a in args:
-                    q.put(str(a))
+                    if label:
+                        q.put((str(a), opts.get('label', '')))
+                    else:
+                        q.put(str(a))
+
+        def plain(self):
+            return True
+
     u = Qui()
-    for k, v in u.configitems('defaults'):
-        u.setconfig('defaults', k, '')
-    return dispatch._dispatch(u, list(args))
+    oldterm = os.environ.get('TERM')
+    os.environ['TERM'] = 'dumb'
+    ret = dispatch._dispatch(u, list(args))
+    if oldterm:
+        os.environ['TERM'] = oldterm
+    return ret
 
 def get_reponame(repo):
     if repo.ui.config('tortoisehg', 'fullpath', False):
@@ -452,3 +490,17 @@ def is_descriptor(obj, attr):
             return hasattr(cls.__dict__[attr], '__get__')
     return None
     
+def export(repo, revs, template='hg-%h.patch', fp=None, switch_parent=False,
+           opts=None):
+    '''
+    export changesets as hg patches.
+    
+    Mercurial moved patch.export to cmdutil.export after version 1.5
+    (change e764f24a45ee in mercurial).
+    '''   
+
+    try:
+        return cmdutil.export(repo, revs, template, fp, switch_parent, opts)
+    except AttributeError:
+        from mercurial import patch
+        return patch.export(repo, revs, template, fp, switch_parent, opts)
